@@ -1,3 +1,4 @@
+import sys
 import six
 from django.db import models
 from django.core.exceptions import ImproperlyConfigured
@@ -21,7 +22,7 @@ class BaseEncryptedField(models.Field):
 
     def __init__(self, *args, **kwargs):
         if not hasattr(settings, 'ENCRYPTED_FIELD_KEYS_DIR'):
-            raise ImproperlyConfigured('You must set the ENCRYPTED_FIELD_KEYS_DIR '
+            raise ImproperlyConfigured('You must set the settings.ENCRYPTED_FIELD_KEYS_DIR '
                                        'setting to your Keyczar keys directory.')
         crypt_class = self.get_crypt_class()
         self.crypt = crypt_class.Read(settings.ENCRYPTED_FIELD_KEYS_DIR)
@@ -29,13 +30,17 @@ class BaseEncryptedField(models.Field):
         # Encrypted size is larger than unencrypted
         self.unencrypted_length = max_length = kwargs.get('max_length', None)
         if max_length:
-            max_length = len(self.prefix) + len(self.crypt.Encrypt('x' * max_length))
-            # TODO: Re-examine if this logic will actually make a large-enough
-            # max-length for unicode strings that have non-ascii characters in them.
-            kwargs['max_length'] = max_length
+            kwargs['max_length'] = self.calculate_crypt_max_length(max_length)
 
         super(BaseEncryptedField, self).__init__(*args, **kwargs)
 
+    def calculate_crypt_max_length(self, unencrypted_length):
+        # TODO: Re-examine if this logic will actually make a large-enough
+        # max-length for unicode strings that have non-ascii characters in them.
+        # For PostGreSQL we might as well always use textfield since there is little
+        # difference (except for length checking) between varchar and text in PG.
+        return len(self.prefix) + len(self.crypt.Encrypt('x' * unencrypted_length))
+
     def get_crypt_class(self):
         """
         Get the Keyczar class to use.
@@ -68,8 +73,9 @@ class BaseEncryptedField(models.Field):
         elif value and (value.startswith(self.prefix)):
             if hasattr(self.crypt, 'Decrypt'):
                 retval = self.crypt.Decrypt(value[len(self.prefix):])
-                if retval:
-                    retval = retval.decode('utf-8')
+                if sys.version_info < (3,):
+                    if retval:
+                        retval = retval.decode('utf-8')
             else:
                 retval = value
         else:
@@ -80,8 +86,9 @@ class BaseEncryptedField(models.Field):
         if value and not value.startswith(self.prefix):
             # We need to encode a unicode string into a byte string, first.
             # keyczar expects a bytestring, not a unicode string.
-            if type(value) == six.types.UnicodeType:
-                value = value.encode('utf-8')
+            if sys.version_info < (3,):
+                if type(value) == six.types.UnicodeType:
+                    value = value.encode('utf-8')
             # Truncated encrypted content is unreadable,
             # so truncate before encryption
             max_length = self.unencrypted_length
@@ -94,6 +101,11 @@ class BaseEncryptedField(models.Field):
             value = self.prefix + self.crypt.Encrypt(value)
         return value
 
+    def deconstruct(self):
+        name, path, args, kwargs = super(BaseEncryptedField, self).deconstruct()
+        kwargs['max_length'] = self.unencrypted_length
+        return name, path, args, kwargs
+
 
 class EncryptedTextField(six.with_metaclass(models.SubfieldBase,
                                             BaseEncryptedField)):
